@@ -230,18 +230,29 @@ def _check_syntax(code, lang, temp_dir, enforce=True):
     """
     # This function needs the language to be explicitly set, so we can
     # generate an appropriate temp filename.
-    if lang == 'python2.7':
+    if lang == 'python2.7' or 'python3' in lang:
         temp_basename = 'inlined_code_from_dxapp_json.py'
     elif lang == 'bash':
         temp_basename = 'inlined_code_from_dxapp_json.sh'
     else:
-        raise ValueError('lang must be one of "python2.7" or "bash"')
+        raise ValueError('lang must be one of "python2.7", "python3", or "bash"')
     # Dump the contents out to a temporary file, then call _check_file_syntax.
     with open(os.path.join(temp_dir, temp_basename), 'w') as ofile:
         ofile.write(code)
     _check_file_syntax(os.path.join(temp_dir, temp_basename), temp_dir, override_lang=lang, enforce=enforce)
 
 
+# convert the error message embedded in the exception to a proper string
+def _error_message_to_string(e, message):
+    if isinstance(message, str):
+        return message
+    elif isinstance(message, bytes):
+        return message.decode("utf-8")
+    else:
+        # What kind of object is this?
+        print("The error message is neither string nor bytes, it is {}".format(type(message)))
+        raise e
+
 def _check_file_syntax(filename, temp_dir, override_lang=None, enforce=True):
     """
     Checks that the code in FILENAME parses, attempting to autodetect
@@ -297,17 +308,17 @@ def _check_file_syntax(filename, temp_dir, override_lang=None, enforce=True):
         if USING_PYTHON2:
             errmsg = e.output
         else:
-            errmsg = e.output.decode("utf-8")
+            errmsg = _error_message_to_string(e, e.output)
         for line in errmsg.strip("\n").split("\n"):
             print("  " + line.rstrip("\n"), file=sys.stderr)
         if enforce:
             raise DXSyntaxError(filename + " has a syntax error")
     except py_compile.PyCompileError as e:
         print(filename + " has a syntax error! Interpreter output:", file=sys.stderr)
-        try:
-            errmsg = e.msg.decode("utf-8")
-        except AttributeError:
+        if USING_PYTHON2:
             errmsg = e.msg
+        else:
+            errmsg = _error_message_to_string(e, e.msg)
         print("  " + errmsg.strip(), file=sys.stderr)
         if enforce:
             raise DXSyntaxError(e.msg.strip())
@@ -664,14 +675,74 @@ def _build_app_remote(mode, src_dir, publish=False, destination_override=None,
         shutil.rmtree(temp_dir)
 
 
+def build_app_from(applet_id, version, publish=False, do_try_update=True, bill_to_override=None,
+                   return_object_dump=False, confirm=True, brief=False, **kwargs):
+
+    applet_desc = dxpy.api.applet_describe(applet_id)
+    app_name = applet_desc["name"]
+    dxpy.executable_builder.verify_developer_rights('app-' + app_name)
+    if not brief:
+        logger.info("Will create app from the applet: %s (%s)" % (applet_desc["name"], applet_desc['id'],))
+
+    applet_region = dxpy.api.project_describe(applet_desc["project"],
+                                              input_params={"fields": {"region": True}})["region"]
+
+    #TODO: make it possible to build multi region app by uploadling
+    # the applet tarball to different regions
+    regional_options = {
+        applet_region: {'applet': applet_id}
+    }
+
+    # Certain metadata is not copied from an applet to the app
+    # It must be passed explicitly otherwise default values will be
+    # set by the API server or an error will be throw during app build
+    # for required non-empty fields
+    fields_to_inherit = (
+        "summary", "title", "description", "developerNotes",
+        "details", "access", "ignoreReuse"
+    )
+    inherited_metadata = {}
+    for field in fields_to_inherit: 
+        if field in applet_desc:
+            inherited_metadata[field] = applet_desc[field]
+
+    required_non_empty = ("summary", "title", "description", "developerNotes")
+    for field in required_non_empty:
+        if field not in inherited_metadata or not inherited_metadata[field]:
+            inherited_metadata[field] = applet_desc["name"]
+    
+    app_id = dxpy.app_builder.create_app_multi_region(regional_options,
+                                                      app_name,
+                                                      None,
+                                                      publish=publish,
+                                                      set_default=publish,
+                                                      billTo=bill_to_override,
+                                                      try_versions=version,
+                                                      try_update=do_try_update,
+                                                      confirm=confirm,
+                                                      inherited_metadata=inherited_metadata,
+                                                      brief=brief)
+    app_describe = dxpy.api.app_describe(app_id)
+
+    if not brief:
+        if publish:
+            logger.info("Uploaded and published app %s/%s (%s) successfully" % (app_describe["name"], app_describe["version"], app_id))
+        else:
+            logger.info("Uploaded app %s/%s (%s) successfully" % (app_describe["name"], app_describe["version"], app_id))
+            logger.info("You can publish this app with: dx publish {n}/{v}".format(n=app_describe["name"], v=app_describe["version"]))
+
+    return app_describe if return_object_dump else {"id": app_id}
+
+
 def build_and_upload_locally(src_dir, mode, overwrite=False, archive=False, publish=False, destination_override=None,
                              version_override=None, bill_to_override=None, use_temp_build_project=True,
                              do_parallel_build=True, do_version_autonumbering=True, do_try_update=True,
                              dx_toolkit_autodep="stable", do_check_syntax=True, dry_run=False,
                              return_object_dump=False, confirm=True, ensure_upload=False, force_symlinks=False,
-                             region=None, **kwargs):
+                             region=None, brief=False, **kwargs):
     dxpy.app_builder.build(src_dir, parallel_build=do_parallel_build)
     app_json = _parse_app_spec(src_dir)
+
     _check_suggestions(app_json, publish=publish)
     _verify_app_source_dir(src_dir, mode, enforce=do_check_syntax)
     if mode == "app" and not dry_run:
@@ -688,8 +759,8 @@ def build_and_upload_locally(src_dir, mode, overwrite=False, archive=False, publ
     if enabled_regions is not None and len(enabled_regions) > 1 and not use_temp_build_project:
         raise dxpy.app_builder.AppBuilderException("Cannot specify --no-temp-build-project when building multi-region apps")
 
+    # Prepare projects in which the app's underlying applets will be built (one per region).
     projects_by_region = None
-
     if mode == "applet" and destination_override:
         working_project, override_folder, override_applet_name = parse_destination(destination_override)
         region = dxpy.api.project_describe(working_project,
@@ -800,7 +871,8 @@ def build_and_upload_locally(src_dir, mode, overwrite=False, archive=False, publ
                 project=project,
                 folder=override_folder,
                 ensure_upload=ensure_upload,
-                force_symlinks=force_symlinks) if not dry_run else []
+                force_symlinks=force_symlinks,
+                brief=brief) if not dry_run else []
 
         # TODO: Clean up these applets if the app build fails.
         applet_ids_by_region = {}
@@ -817,6 +889,7 @@ def build_and_upload_locally(src_dir, mode, overwrite=False, archive=False, publ
                     override_name=override_applet_name,
                     dx_toolkit_autodep=dx_toolkit_autodep,
                     dry_run=dry_run,
+                    brief=brief,
                     **kwargs)
                 if not dry_run:
                     logger.debug("Created applet " + applet_id + " successfully")
@@ -876,17 +949,17 @@ def build_and_upload_locally(src_dir, mode, overwrite=False, archive=False, publ
                                                               billTo=bill_to_override,
                                                               try_versions=try_versions,
                                                               try_update=do_try_update,
-                                                              confirm=confirm)
+                                                              confirm=confirm,
+                                                              brief=brief)
 
             app_describe = dxpy.api.app_describe(app_id)
 
-            if publish:
-                print("Uploaded and published app %s/%s (%s) successfully" % (app_describe["name"], app_describe["version"], app_id), file=sys.stderr)
-            else:
-                print("Uploaded app %s/%s (%s) successfully" % (app_describe["name"], app_describe["version"], app_id), file=sys.stderr)
-                print("You can publish this app with:", file=sys.stderr)
-                print("  dx publish {n}/{v}".format(n=app_describe["name"],
-                                                    v=app_describe["version"]), file=sys.stderr)
+            if not brief:
+                if publish:
+                    logger.info("Uploaded and published app %s/%s (%s) successfully" % (app_describe["name"], app_describe["version"], app_id))
+                else:
+                    logger.info("Uploaded app %s/%s (%s) successfully" % (app_describe["name"], app_describe["version"], app_id))
+                    logger.info("You can publish this app with: dx publish {n}/{v}".format(n=app_describe["name"], v=app_describe["version"]))
 
             return app_describe if return_object_dump else {"id": app_id}
 
@@ -909,6 +982,34 @@ def _build_app(args, extra_args):
 
     """
 
+    if args._from:
+        # BUILD FROM EXISTING APPLET
+        try:
+            output = build_app_from(
+                args._from,
+                [args.version_override],
+                publish=args.publish,
+                do_try_update=args.update,
+                bill_to_override=args.bill_to,
+                confirm=args.confirm,
+                return_object_dump=args.json,
+                brief=args.brief,
+                **extra_args
+            )
+            if output is not None and args.run is None:
+                print(json.dumps(output))
+
+        except dxpy.app_builder.AppBuilderException as e:
+            # AppBuilderException represents errors during app building
+            # that could reasonably have been anticipated by the user.
+            print("Error: %s" % (e.args,), file=sys.stderr)
+            sys.exit(3)
+        except dxpy.exceptions.DXAPIError as e:
+            print("Error: %s" % (e,), file=sys.stderr)
+            sys.exit(3)
+
+        return output['id']
+
     if not args.remote:
         # LOCAL BUILD
 
@@ -934,6 +1035,7 @@ def _build_app(args, extra_args):
                 confirm=args.confirm,
                 return_object_dump=args.json,
                 region=args.region,
+                brief=args.brief,
                 **extra_args
                 )
 
